Attribute VB_Name = "modCore"
Option Explicit
' ********************************************************************************                                                                             *
' *                                                                              *
' *  modCore.bas - core.exe this is the main core module                         *
' *                                                                              *
' *  Copyright (c) 2000-2001 Asperon (asperon@home.se), last updated 2001-05-31  *
' *                                                                              *
' *  This program is free software; you can redistribute it and/or modify        *
' *  it under the terms of the GNU General Public License as published by        *
' *  the Free Software Foundation; either version 2 of the License, or           *
' *  any later version.                                                          *
' *                                                                              *
' *  This program is distributed in the hope that it will be useful,             *
' *  but WITHOUT ANY WARRANTY; without even the implied warranty of              *
' *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
' *  GNU General Public License for more details.                                *
' *                                                                              *
' *  You should have received a copy of the GNU General Public License           *
' *  along with this program; if not, write to the Free Software                 *
' *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   *
' *                                                                              *
' ********************************************************************************

Public bExit As Boolean
Public bReload As Boolean
Public bColors As Boolean
Private hwnd As Long
'   This is the main procedure, from this everything is called.
Sub Main()
    
    Dim textline As String
    Dim I As Integer
    Dim ret As Long
    Dim sections() As String
    
    On Error Resume Next
    
    ' We only allow one instance of core at any time
    If App.PrevInstance Then End

    ' If core needs to reload, this is where we start up.
start:
    
    ' Reset the module sections
    ReDim sections(0)
    
    ' Reset the log
    ReDim Log(0)
    
    ' Reset the Bangs
    ReDim Bangs(0)
    
    ' Reset the Plugins
    ReDim Plugins(0)
    
    ' Reset the Colors
    ReDim SysColors(0)
    
    ' Create a new instance of CServer
    Set oServer = New core.CServer
    
    ' Load the settings module. This is used by core to access its settings.
    Set oSettings = CreateObject("settings.CSettings")
    If oSettings Is Nothing Then
        If Err.Number = 429 Then
            ret = oServer.RegisterComponent(App.path & "\settings.dll", DllRegisterServer)
            If ret = 0 Then
                Set oSettings = CreateObject("settings.CSettings")
            Else
                MsgBox "Couldn't load settings.dll, this module is needed to run core.", vbCritical, "error loading settings.dll"
                End
                Exit Sub
            End If
        Else
            MsgBox "There was a unknown error loading settings.dll, the error was (" & Err.Number & ") " & Err.Description & ".", vbCritical, "error loading settings.dll"
            End
            Exit Sub
        End If
    End If
    
    ' Initiate the settings module with any commandline parameters passed to us.
    oSettings.InitSettings command
    
    ' Initiate the system colors
    If oServer.evalBool(oSettings.GetKey("core", "usecolors")) Then
        bColors = True
        SetupSystemColors
    End If
    bExit = False
    bReload = False
    
    ' Load the modules
    sections = oSettings.GetSections
    If UBound(sections) = 0 Then
            MsgBox "No modules found to load, running core without modules is of no use.", vbCritical, "No modules found"
            bExit = True
    Else
        For I = 1 To UBound(sections)
            textline = Trim(oServer.GetKey(sections(I), "module"))
            If textline <> "" Then
                ret = oServer.LoadPlugin(sections(I), textline)
                If ret > 0 Then
                    oServer.Execute "!log Error loading " & textline & " as " & sections(I) & " " & Err.Description
                End If
            End If
        Next I
    End If
    
    ' Load the about form, we need it to subclass
    Load frmAbout
    
    'Start subclassing
    hwnd = SetWindowLong(frmAbout.hwnd, GWL_WNDPROC, GetAddress(AddressOf WndProc))
    
    While Not bExit
        Call Sleep(100)
        DoEvents
    Wend
    
    ' Stop subclassing
    SetWindowLong frmAbout.hwnd, GWL_WNDPROC, hwnd
    
    If bReload Then
        oServer.Execute "!log core is reloading."
    Else
        oServer.Execute "!log core is closing down."
    End If
    Unload frmAbout
    Call oServer.UnloadPlugins
    If bColors Then
        RestoreColors
    End If
    Set oSettings = Nothing
    Set oServer = Nothing
    
    If bReload Then
        GoTo start
    Else
        End
    End If

End Sub
Private Function WndProc(ByVal hwnd As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

    Select Case uMsg
    Case WM_QUERYENDSESSION:
        oServer.Execute "!log Windows is shutting down!"
        WndProc = True
    Case WM_ENDSESSION:
        bExit = True
    End Select

    WndProc = DefWindowProc(hwnd, uMsg, wParam, lParam)

End Function
Private Function GetAddress(ByVal lngAddr As Long) As Long
    ''Used with AddressOf to return the address in memory of a procedure.

    GetAddress = lngAddr&
    
End Function
